﻿using System;
using System.Globalization;
using System.Net.Sockets;

namespace SatIp.RtspSample.Rtsp
{
    /// <summary>
  /// A simple implementation of an RTSP client.
  /// </summary>
    public class RtspClient : IDisposable
  {
    #region variables

    private string _serverHost = null;
    private int _serverPort = -1;
    private TcpClient _client = null;
    private int _cseq = 1;
    private object _lockObject = new object();

    #endregion

    /// <summary>
    /// Initialise a new instance of the <see cref="RtspClient"/> class.
    /// </summary>
    /// <param name="serverHost">The RTSP server host name or IP address.</param>
    /// <param name="serverPort">The port on which the RTSP server is listening.</param>
    public RtspClient(string serverHost)
    {
      _serverHost = serverHost;
      _serverPort = 554;
    }

    ~RtspClient()
    {
      Dispose(false);
    }

    /// <summary>
    /// Send an RTSP request and retrieve the response.
    /// </summary>
    /// <param name="request">The request.</param>
    /// <param name="response">The response.</param>
    /// <returns>the response status code</returns>
    public RtspStatusCode SendRequest(RtspRequest request, out RtspResponse response)
    {
      response = null;
      lock (_lockObject)
      {
        NetworkStream stream = null;
        int retryCount = 0;
        while (true)
        {
          try
          {
            if (_client == null)
            {
              _client = new TcpClient(_serverHost, _serverPort);
            }
            else if (!_client.Connected)
            {
              _client.Connect(_serverHost, _serverPort);
            }
          }
          catch (Exception ex)
          {
            //this.LogError(ex, "RTSP: failed to connect to server, host = {0}, port = {1}", _serverHost, _serverPort);
            return RtspStatusCode.RequestTimeOut;
          }
          try
          {
            stream = _client.GetStream();
            if (stream == null)
            {
              throw new Exception();
            }
            break;
          }
          catch (Exception ex)
          {
            _client.Close();
            _client = null;
            if (retryCount == 1)
            {
              //this.LogError(ex, "RTSP: failed to open stream to server, host = {0}, port = {1}", _serverHost, _serverPort);
              return RtspStatusCode.RequestTimeOut;
            }
            retryCount++;
          }
        }

        try
        {
          // Send the request and get the response.
          request.Headers.Add("CSeq", _cseq.ToString());
          _cseq++;
          byte[] requestBytes = request.Serialise();
          stream.Write(requestBytes, 0, requestBytes.Length);
          byte[] responseBytes = new byte[_client.ReceiveBufferSize];
          int byteCount = stream.Read(responseBytes, 0, responseBytes.Length);
          response = RtspResponse.Deserialise(responseBytes, byteCount);

          // Did we get the whole response?
          string contentLengthString;
          int contentLength = 0;
          if (response.Headers.TryGetValue("Content-Length", out contentLengthString))
          {
            contentLength = int.Parse(contentLengthString);
            if ((string.IsNullOrEmpty(response.Body) && contentLength > 0) || response.Body.Length < contentLength)
            {
              if (response.Body == null)
              {
                response.Body = string.Empty;
              }
              while (byteCount > 0 && response.Body.Length < contentLength)
              {
                byteCount = stream.Read(responseBytes, 0, responseBytes.Length);
                response.Body += System.Text.Encoding.UTF8.GetString(responseBytes, 0, byteCount);
              }
            }
          }
          return response.StatusCode;
        }
        finally
        {
          stream.Close();
          stream.Dispose();
        }
      }
    }

    #region IDisposable member

    /// <summary>
    /// Release and dispose all resources.
    /// </summary>
    public void Dispose()
    {
      Dispose(true);
      GC.SuppressFinalize(this);
    }

    /// <summary>
    /// Release and dispose all resources.
    /// </summary>
    /// <param name="isDisposing"><c>True</c> if the instance is being disposed.</param>
    private void Dispose(bool isDisposing)
    {
      if (isDisposing)
      {
        lock (_lockObject)
        {
          if (_client != null)
          {
            _client.Close();
            _client = null;
          }
        }
      }
    }

    #endregion
  }
}
